@@ -172,6 +172,113 @@ def execute_search_command(
         "messages": messages,
         "output": result[0] + f"\n\n\n Citation Links: {result[1]}",
     }
+    
+
+
+def get_facts_for_rag(
+    kuzu_db_path: str,
+    chroma_db_path: str,
+    query: str,
+    group_filters: Optional[List[str]] = None,
+    top_k: int = 10,
+) -> str:
+    """Get facts for RAG by combining vector and graph search
+
+    Args:
+        kuzu_db_path: Path to Kuzu graph database
+        chroma_db_path: Path to Chroma vector database
+        query: Search query
+        group_filters: Optional list of groups to filter by
+        top_k: Number of results to return
+        embedding_model: Model to use for embeddings
+        provider: Provider for embeddings
+
+    Returns:
+        Formatted context string with retrieved facts
+    """
+    # Initialize connections
+    kuzu_conn = init_db(kuzu_db_path)
+    chroma_client, chroma_collection = setup_chroma_db( 
+        "knowledge_graph",
+        "Facts extracted from various sources",
+        chroma_db_path
+    )
+
+    # Perform hybrid search
+    results = hybrid_search_with_chroma(
+        kuzu_conn=kuzu_conn,
+        chroma_collection=chroma_collection,
+        query=query,
+        group_filter=group_filters,
+        top_k=top_k,
+    )
+
+    # Format results as context for RAG
+    context = "Related facts:\n\n"
+
+    # First include direct vector matches
+    context += "Most relevant facts:\n"
+    vector_matches = [r for r in results if r["source"] == "vector_search"]
+    for i, item in enumerate(vector_matches):
+        context += f"{i+1}. {item['fact']}\n"
+
+    # Then include graph-related facts
+    context += "\nRelated concepts:\n"
+    graph_matches = [r for r in results if r["source"] != "vector_search"]
+    for i, item in enumerate(graph_matches):
+        group = item["source"].replace("graph_relation_via_", "")
+        context += f"{i+1}. {item['fact']} (related via {group})\n"
+
+    # Close connections
+    kuzu_conn.close()
+
+    return context
+def answer_with_rag(
+    query: str,
+    kuzu_db_path,
+    chroma_db_path,
+    model,
+    provider,    
+) -> str:
+    """Answer a query using RAG with facts from the knowledge base
+
+    Args:
+        query: User query
+        kuzu_db_path: Path to Kuzu graph database
+        chroma_db_path: Path to Chroma vector database
+        model: LLM model to use
+        provider: LLM provider
+        embedding_model: Model to use for embeddings
+
+    Returns:
+        Answer from the model
+    """
+    # Get relevant facts using hybrid search
+    context = get_facts_for_rag(
+        kuzu_db_path,
+        chroma_db_path,
+        query,
+    )
+
+    # Craft prompt with retrieved context
+    prompt = f"""
+    Answer this question based on the retrieved information.
+
+    Question: {query}
+
+    {context}
+
+    Please provide a comprehensive answer based on the facts above. If the information
+    doesn't contain a direct answer, please indicate that clearly but try to synthesize
+    from the available facts.
+    """
+
+    # Get response from LLM
+    response = get_llm_response(prompt, model=model, provider=provider)
+
+    return response["response"]
+
+    
 def execute_rag_command(
     command: str,
     vector_db_path: str, 
@@ -330,9 +437,6 @@ def execute_rag_command(
         
 def execute_brainblast_command(
     command: str,
-    command_history,
-    messages=None,
-    top_k: int = 5,  # Fewer results per chunk to keep total manageable
     **kwargs
 ) -> dict:
     """
@@ -490,7 +594,6 @@ def execute_brainblast_command(
         # Get LLM response
         response = get_llm_response(
             prompt,
-            messages=messages,
             **kwargs
         )
         return response
